#-
* This file is part of aITALC
* Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>
*------------------------------------------------------------- 
** <joinmm.frm> designed to pick up the kinematical information out of DIANA and
** reproduce the Matrix Elements and differential cross section subroutines.
** It calculates explicitely the product of 2 fermion chains.
*-------------------------------------------------------------------------------
#include Kit.inc
#include model.inc
#include aITALC.inc
#include `processname'.in #exts1
#include `processname'.in #topologies0
#define code "15"

NF FER,FERB;
auto S m,u;
NF H,Hc;


#if (`LEGS' == 4 ) || ( `LEGS' == 3)
*-------------------------------------------------------------------------------
* Leaving the possibility to work in a Right-Left instead of Gamma,Gamma5
#if `code' == 15
  #define a "1"
  #define b "5"
#elseif `code' == lr
  #define a "l"
  #define b "r"
#endif
*-------------------------------------------------------------------------------
* Let's define the big chains, for all possible fermion channels
#do ch1={`fermioncurrentsnames'}
  #do ch2={`fermioncurrentsnames'}
#do i1=1,9
  #do i2=1,4
    #do j1=1,9
      #do j2=1,4
L [`ch1'`ch2',`i1',`i2',`j1',`j2'] = Hc(`ch1',`i1',`i2')*H(`ch2',`j1',`j2');
      #enddo
    #enddo
  #enddo
#enddo
  #enddo
#enddo


#do k=1,4

#if     `code' == 15
  #if     `k' == 1
    #define k1 "1"
    #define k2 "1" 
  #elseif `k' == 2
    #define k1 "1"
    #define k2 "5"
  #elseif `k' == 3
    #define k1 "5"
    #define k2 "1"
  #elseif `k' == 4
    #define k1 "5"
    #define k2 "5"
  #endif
#elseif `code' == lr
  #if     `k' == 1
    #define k1 "l"
    #define k2 "l" 
  #elseif `k' == 2
    #define k1 "l"
    #define k2 "r"
  #elseif `k' == 3
    #define k1 "r"
    #define k2 "l"
  #elseif `k' == 4
    #define k1 "r"
    #define k2 "r"
  #endif
#endif

** The FER and FERB functions will represent the spinors of fermions
** They have 3 arguments, 1st is the fermion line they belong to,
** 2nd is a variable telling us whether it is fermion (antifermion)
** +1 (-1). Last one is the mass.

** Let's make the index expansion of different kind of matrix elements
  #do index=1,9
    #do ch={`fermioncurrentsnames'}

      id H(`ch',`index',`k')=
      #do ic=1,`EXTFERMICURRENTS'
        #redefine ipart "`IFFO`ch'{`ic'}'"
        FER(`ic',u{`ipart'},m{`ipart'})*FERB(`ic',u{`ipart'},m{`ipart'})*defgam(`ic',`ch',`index',`k{`ic'}')*
      #enddo
                             1;

      id Hc(`ch',`index',`k')=
      #do ic=1,`EXTFERMICURRENTS'
        #redefine ipart "`IFFI`ch'{`ic'}'"
        FER(`ic',u{`ipart'},m{`ipart'})*FERB(`ic',u{`ipart'},m{`ipart'})*defgamad(`ic',`ch',`index',`k{`ic'}')*
      #enddo
                             1;
    #enddo
* Of channel
  #enddo
* Of index
#enddo
* Of k

*-------------------------------------------------------------------------------
** Commutation of different fermion lines means we have to order them, before
** equaling traces, otherwise mixing channel terms who belong to iline=0 
** show up a wrong ordering.

repeat;
id FER?(2,?a)*FERB?(1,?b)= FERB(1,?b)*FER(2,?a);
endrepeat;

*-------------------------------------------------------------------------------
** Now we identify the definitions of our gamma chains
** Fortunately, channel (`ch') S and T share definitions

*Scalar type
#do ch={`fermioncurrentsnames'}

id defgam(1,`ch',1,a?)=                 gam(1,a);
id defgam(2,`ch',1,a?)=                 gam(2,a);

id defgam(1,`ch',2,a?)=                 gam(1,a);
id defgam(2,`ch',2,a?)= gam(2,`P`ch'2')*gam(2,a);

id defgam(1,`ch',3,a?)= gam(1,`P`ch'1')*gam(1,a);
id defgam(2,`ch',3,a?)=                 gam(2,a);

id defgam(1,`ch',4,a?)= gam(1,`P`ch'1')*gam(1,a);
id defgam(2,`ch',4,a?)= gam(2,`P`ch'2')*gam(2,a);


id defgamad(1,`ch',1,a?)=               gamad(1,a);
id defgamad(2,`ch',1,a?)=               gamad(2,a);

id defgamad(1,`ch',2,a?)=               gamad(1,a);
id defgamad(2,`ch',2,a?)= gam(2,`P`ch'2')*gam(2,a);

id defgamad(1,`ch',3,a?)= gam(1,`P`ch'1')*gam(1,a);
id defgamad(2,`ch',3,a?)=               gamad(2,a);

id defgamad(1,`ch',4,a?)= gam(1,`P`ch'1')*gam(1,a);
id defgamad(2,`ch',4,a?)= gam(2,`P`ch'2')*gam(2,a);

*----------------------------------------------------------
*Vector type
id defgam(1,`ch',5,a?)= gam(1,mu1)         *gam(1,a);
id defgam(2,`ch',5,a?)= gam(2,mu1)         *gam(2,a);

id defgam(1,`ch',6,a?)= gam(1,mu1)         *gam(1,a);
id defgam(2,`ch',6,a?)= gam(2,mu1,`P`ch'2')*gam(2,a);

id defgam(1,`ch',7,a?)= gam(1,mu1,`P`ch'1')*gam(1,a);
id defgam(2,`ch',7,a?)= gam(2,mu1)         *gam(2,a);

id defgam(1,`ch',8,a?)= gam(1,mu1,`P`ch'1')*gam(1,a);
id defgam(2,`ch',8,a?)= gam(2,mu1,`P`ch'2')*gam(2,a);


id defgamad(1,`ch',5,a?)= gam(1,nu1)           *gam(1,a);
id defgamad(2,`ch',5,a?)= gam(2,nu1)           *gam(2,a);

id defgamad(1,`ch',6,a?)= gam(1,nu1)           *gam(1,a);
id defgamad(2,`ch',6,a?)= gam(2,`P`ch'2',nu1)*gamad(2,a);

id defgamad(1,`ch',7,a?)= gam(1,`P`ch'1',nu1)*gamad(1,a);
id defgamad(2,`ch',7,a?)= gam(2,nu1)           *gam(2,a);

id defgamad(1,`ch',8,a?)= gam(1,`P`ch'1',nu1)*gamad(1,a);
id defgamad(2,`ch',8,a?)= gam(2,`P`ch'2',nu1)*gamad(2,a);

*----------------------------------------------------------
*Tensor rank-2 type
id defgam(iline?,`ch',9,a?)= gam(iline,mu1,mu2)*gam(iline,a);

id defgamad(iline?,`ch',9,a?)= gam(iline,nu2,nu1)*gamad(iline,a);

#enddo

.sort

*-------------------------------------------------------------------------------
** Now for the mixing ST and TS or so we have to equal the index lines 1 and 2.
** First we avoid substitution on SS TT and UU

#do i1=1,9
  #do i2=1,4
    #do j1=1,9
      #do j2=1,4
        #do ch={`fermioncurrentsnames'}
          hide [`ch'`ch',`i1',`i2',`j1',`j2'];
        #enddo
      #enddo
    #enddo
  #enddo
#enddo

** The main sentence for all functions with index line
id FER?(iline?,?a)=FER(0,?a);
.sort
*-------------------------------------------------------------------------------
unhide;
** If we want them inmediately to be active for printing sort them,
*.sort

** Color sum and average on states (without color (gluons) exchange)
** For formulae, check page 142/143 of alorca's doctorand book "Loop's and techniques"
** Basically is an extension of the multiplicative color factor in the cross section.

if (match(FER(0,u1?,m1?)));
* Case of interference; combinatorial factor = 1/(color any ext fermion)
 multiply 1/`color1';
elseif (match(FER(iline?$finalline3,u3,m3)) && match(FER(iline?$finalline4,u4,m4)));
 if ($finalline3 == $finalline4);
* Case of SSdagger, since they (particles 3,4) share the same fermion line index
* and it's different from 0 (reserved for interference terms)  
  multiply `color3'/`color1';
 else;
* Case of TTdagger and UUdagger, they multiply by 1 for all cases.
 endif;
endif;


** Spin average and sum
id FER(iline?,u1,m1)*FERB(iline?,u1,m1)=1/2*(g_(iline,`absep1')+u1*gi_(iline)*m1);
id FER(iline?,u2,m2)*FERB(iline?,u2,m2)=1/2*(g_(iline,`absep2')+u2*gi_(iline)*m2);
id FER(iline?,u3,m3)*FERB(iline?,u3,m3)=  -1*g_(iline,`absep3')+u3*gi_(iline)*m3 ;
id FER(iline?,u4,m4)*FERB(iline?,u4,m4)=  -1*g_(iline,`absep4')+u4*gi_(iline)*m4 ;

*print [ST,5,1,5,1];
*print +s [SS,5,1,5,1];
*print [TT,5,1,5,1];
*.end


** gammas to g_
id gam(iline?,1)= gi_(iline);
id gam(iline?,5)= g5_(iline);
id gam(iline?,r)= g6_(iline)/2;
id gam(iline?,l)= g7_(iline)/2;

id gamad(iline?,1)= gi_(iline);
id gamad(iline?,5)=-g5_(iline);
id gamad(iline?,r)= g7_(iline)/2;
id gamad(iline?,l)= g6_(iline)/2;


id gam(iline?,?a)=g_(iline,?a);
.sort

*-------------------------------------------------------------------------------
** Now we trace the different fermion lines
Trace4 1;
Trace4 0;
Trace4 2;

*-------------------------------------------------------------------------------
** Because the introduction of epsilon tensor e_ is done automatically, we
** don't have to care about i factors or so, we just contract them

contract;
** Now epsilon vector contractions for the 4 non-indepentent momenta:
id e_(p1,p2,p3,p4)=0;

** We also apply dimensionality, since the ffactors are UV finite by themselves
** after summing the different topologies
id D=4;

.sort

*-------------------------------------------------------------------------------
** The momenta renaming into Mandelstam

id p1?.p2? = sp(p1,p2);

#do j=1,4
  id sp(`absep{`j'}',`absep{`j'}')=m`j'^2;
#enddo

id sp(`absep1',`absep2')=1/2*(sman-m1^2-m2^2);
id sp(`absep1',`absep3')=1/2*(tman-m1^2-m3^2);
id sp(`absep1',`absep4')=1/2*(uman-m1^2-m4^2);
id sp(`absep2',`absep1')=1/2*(sman-m2^2-m1^2);
id sp(`absep2',`absep3')=1/2*(uman-m2^2-m3^2);
id sp(`absep2',`absep4')=1/2*(tman-m2^2-m4^2);
id sp(`absep3',`absep1')=1/2*(tman-m3^2-m1^2);
id sp(`absep3',`absep2')=1/2*(uman-m3^2-m2^2);
id sp(`absep3',`absep4')=1/2*(sman-m3^2-m4^2);
id sp(`absep4',`absep1')=1/2*(uman-m4^2-m1^2);
id sp(`absep4',`absep2')=1/2*(tman-m4^2-m2^2);
id sp(`absep4',`absep3')=1/2*(sman-m4^2-m3^2);

** And some convention about which variable should be dropped out!

#if (({`fermioncurrentsnames'} == {S,T}) || ({`fermioncurrentsnames'} == {S})   || ({`fermioncurrentsnames'} == {T}))
  id uman=-sman-tman+m1^2+m2^2+m3^2+m4^2;
#elseif (({`fermioncurrentsnames'} == {S,U}) || ({`fermioncurrentsnames'} == {U}))
  id tman=-sman-uman+m1^2+m2^2+m3^2+m4^2;
#elseif {`fermioncurrentsnames'} == {T,U}
  id sman=-tman-uman+m1^2+m2^2+m3^2+m4^2;
#endif

.sort
*-------------------------------------------------------------------------------
** Finally we identify the abstract m1..m4, u1..u4

#do ipart=1,`LEGS'
  id m`ipart'=`em{`ipart'}';
  id u`ipart'=`u{`ipart'}';
#enddo

** Possibility to neglect some masses defined in Neglectedmasses.inc
#call neglectmass(0)


format rational spaces;
print +s +f;
.sort

format doublefortran;
.sort
*-------------------------------------------------------------------------------
* Writing out the fortrans


#do ch1={`fermioncurrentsnames'}
 #do ch2={`fermioncurrentsnames'}
#write <matrixelements`ch1'`ch2'.F> "C This file <matrixelements`ch1'`ch2'.F> is part of aITALC"
#write <matrixelements`ch1'`ch2'.F> "C Generated automatically with FORM3"
#write <matrixelements`ch1'`ch2'.F> "C Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>"
#write <matrixelements`ch1'`ch2'.F> "C ----------------------------------------------------------------------"
#write <matrixelements`ch1'`ch2'.F> "      subroutine M_`ch1'`ch2'(mm_`ch1'`ch2')"
#write <matrixelements`ch1'`ch2'.F> ""
#write <matrixelements`ch1'`ch2'.F> "      implicit none"
#write <matrixelements`ch1'`ch2'.F> "      double precision mm_`ch1'`ch2'(9,4,9,4)"
#write <matrixelements`ch1'`ch2'.F> "#include <variablelist.hf>"

#do j2=1,4
 #do j1=1,9
  #do i2=1,4
#write <matrixelements`ch1'`ch2'.F> "C ----------------------------------------------------------------------"
   #do i1=1,9
#write <matrixelements`ch1'`ch2'.F> "      mm_`ch1'`ch2'(`i1',`i2',`j1',`j2') = %e",[`ch1'`ch2',`i1',`i2',`j1',`j2'](mm_`ch1'`ch2'(`i1',`i2',`j1',`j2'))
*#message [joinmm: writing the term `ch1'`ch2',`i1',`i2',`j1',`j2']
     #enddo
    #enddo
   #enddo
  #enddo
#write <matrixelements`ch1'`ch2'.F> "C ----------------------------------------------------------------------"
#write <matrixelements`ch1'`ch2'.F> ""
#write <matrixelements`ch1'`ch2'.F> "      return"
#write <matrixelements`ch1'`ch2'.F> "      end"
 #enddo
#enddo

#elseif `LEGS' == 2
** For selfenergies we don't build up anything
 #write <matrixelements.F> ""
#else
 #message [joinmm: 5 legs or more not considered]
#endif

.end

*$Id: joinmm.frm,v 1.13 2004/10/11 18:09:16 alorca Exp $
