C File <c0wdd0.F> containing the C0 function for one propagator with
C finite width and one with photon.
C This file is part of aITALC
C Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>
C-----------------------------------------------------------------------
      double complex function C0wcc0(
     &     P1Sin,P2Sin,P1P2Sin, !external momenta
     &     cM1Sin,cM2Sin,cM3Sin) !internal masses
C-----------------------------------------------------------------------
      implicit none
      integer index
      double precision P1Sin,P2Sin,P1P2Sin
      double complex cM1Sin,cM2Sin,cM3Sin
      double precision P1S,P2S,P3S,P4S,P1P2S,P2P3S
      double precision M1S
      double complex cM2S,cM3S

      double precision IG3,IG2
      integer iy,iz,icase
      double complex y(2),z(2),ly(2),ydiff
      double precision a(0:2),dy(2)
      integer sy(2)
      double complex c(0:2)
      double complex m0,x2,x3
      double precision m1,m4,m2,m3,s,t,xs,lambda
      double complex lambdapart,part1,part2,part3
      double precision dcondition,thresh1,thresh2
      double precision DDILOG
      double complex CDLOGIE,DILOGIE,CDDILOG,CDDILOGMU
c#define DEBUG 1
C-----------------------------------------------------------------------
C Now we have to apply symmetry properties to place the photon in the cM1S

C The photon will be in the naked integration momenta => DBLE(M1S).EQ.0
C And the Complex valued boson on the next => DIMAG(M2S).NE.0
c      print *,'cM1Sin',cM1Sin
c      print *,'cM2Sin',cM2Sin
c      print *,'cM3Sin',cM3Sin

      if (DBLE(cM1Sin).EQ.0d0) then
C Good, we don't need to rotate or inverse
#ifdef DEBUG
        print *,'c0wcc0: step 1'
#endif
        M1S=DBLE(cM1Sin)
        cM2S=cM2Sin
        cM3S=cM3Sin
        
        P1S=P1Sin
        P2S=P2Sin
        P1P2S=P1P2Sin
      elseif (DBLE(cM2Sin).EQ.0d0) then
C Only one rotation
#ifdef DEBUG
        print *,'c0wcc0: step 3'
#endif
        M1S=DBLE(cM2Sin)
        cM2S=cM3Sin
        cM3S=cM1Sin

        P1S=P2Sin
        P2S=P1P2Sin
        P1P2S=P1Sin
      elseif (DBLE(cM3Sin).EQ.0d0) then
C Only one rotation more
#ifdef DEBUG
        print *,'c0wcc0: step 5'
#endif
        M1S=DBLE(cM3Sin)
        cM2S=cM1Sin
        cM3S=cM2Sin

        P1S=P1P2Sin
        P2S=P1Sin
        P1P2S=P2Sin
      endif
C-----------------------------------------------------------------------
C In order to compute the C0 function we need the roots of two polynomes
C one of complex nature and the other real.

      a(2)=P2S
      a(1)=P1S-P1P2S-P2S
      a(0)=P1P2S
      c(2)=DCMPLX(a(2))
      c(1)=cM2S-P2S-cM3S
      c(0)=cM3S

      if (P2S+P1P2S.EQ.0D0) then
        thresh2=P1S*1d100
      else
        thresh2=P1S/(DSQRT(P2S)+DSQRT(P1P2S))**2
        if ((DSQRT(P2S)-DSQRT(P1P2S)).NE.0d0) then
          thresh1=P1S/(DSQRT(P2S)-DSQRT(P1P2S))**2
        else
          thresh1=P1S*1d100
        endif
      endif
C      print *,'thresh1=',thresh1
C      print *,'thresh2=',thresh2
C      stop
      if ((DABS(1-thresh1).LT.2d-15).OR.(DABS(1-thresh2).LT.2d-15)) then
        print *,'c0wcc0: threshold cases not yet computed'
        stop
c      elseif (P1S.LT.0d0) then
C Propagator don't develop imaginary part, comeback to looptools.
c        goto 10
      elseif ((thresh2.GT.1d0).OR.(thresh1.LT.1d0)) then
C Real roots, real coefficients
        call DGETROOTS(a(2),a(1),a(0),y(1),y(2),ydiff,icase)
c       print *,'roots(',icase,')=',y(1),y(2)
        dy(1)=DBLE(y(1))
        dy(2)=DBLE(y(2))
        sy(1)=-1
        sy(2)=+1
C The signs of the im*epsilon parts, always first root negative
C Complex roots, complex c(1)
        call CGETROOTS(c(2),c(1),c(0),z(1),z(2))
c        print *,'croots=',z(1),z(2)
        
        C0wcc0=DCMPLX(0)
        do iy=1,2
          if ((dy(iy).GT.0D0).AND.(dy(iy).LT.1d0)) then
            ly(iy)=CDLOGIE((dy(iy)-1)/dy(iy),sy(iy))
          else
            ly(iy)=DLOG((dy(iy)-1)/dy(iy))
          endif
c          print *,'ly(',iy,')=',ly(iy)
          C0wcc0=C0wcc0+sy(iy)*(0
     &         -ly(iy)*LOG(-(P1S-cM2S-P3S+cM3S)/P2S)
     &         -DILOGIE(1/dy(iy),-sy(iy))
     &         )
          do iz=1,2
           C0wcc0=C0wcc0+sy(iy)*(0
     &           +ly(iy)*LOG(y(iy)-z(iz))
     &           +CDDILOGMU(y(iy)/(y(iy)-z(iz)),1d0,sy(iy))
     &           -CDDILOGMU((y(iy)-1)/(y(iy)-z(iz)),1d0,sy(iy))
     &           )
          enddo
        enddo

      elseif ((thresh2.LT.1d0).AND.(thresh1.GT.1d0)) then
C Complex roots, real coefficients
        call DGETROOTS(a(2),a(1),a(0),y(1),y(2),ydiff,icase)
C Complex roots, complex c(1)
        call CGETROOTS(c(2),c(1),c(0),z(1),z(2))
        C0wcc0=DCMPLX(0)
        do iy=1,2

          ly(iy)=LOG((y(iy)-1)/y(iy))

          C0wcc0=C0wcc0+sy(iy)*(0
     &         -ly(iy)*LOG(-(P1S-cM2S)/P2S)
     &         -CDDILOG(1/y(iy))
     &         )
          do iz=1,2
           C0wcc0=C0wcc0+sy(iy)*(0
     &           +ly(iy)*LOG(y(iy)-z(iz))
     &           +CDDILOG(y(iy)/(y(iy)-z(iz)))
     &           -CDDILOG((y(iy)-1)/(y(iy)-z(iz)))
     &           )
          enddo
        enddo
c        print *,'c0wcc0: complex roots not yet checked!'
      else
        print *,'c0wcc0: How is another case possible?, please check!'
        stop
      endif

      C0wcc0=-C0wcc0/ydiff/P2S
c      print *,'c0=',C0wcc0
C-----------------------------------------------------------------------
      return
      end

CC $Id: c0wcc0.F,v 1.6 2004/10/13 12:53:34 alorca Exp $
